import stripe
from stripe import api_requestor, six, util
from stripe.stripe_object import StripeObject

from stripe.six.moves.urllib.parse import quote_plus


async def list_patch(
    self, api_key=None, stripe_version=None, stripe_account=None, **params
):
    stripe_object = await self._request(
        "get",
        self.get("url"),
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        **params
    )
    stripe_object._retrieve_params = params
    return stripe_object

async def create_patch(
    self,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    return await self._request(
        "post",
        self.get("url"),
        api_key=api_key,
        idempotency_key=idempotency_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        **params
    )

async def retrieve_patch(
    self,
    id,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    url = "%s/%s" % (self.get("url"), quote_plus(util.utf8(id)))
    return await self._request(
        "get",
        url,
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        **params
    )

async def _request_patch(
    self,
    method_,
    url_,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    api_key = api_key or self.api_key
    stripe_version = stripe_version or self.stripe_version
    stripe_account = stripe_account or self.stripe_account

    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    headers = util.populate_headers(idempotency_key)
    response, api_key = await requestor.request(method_, url_, params, headers)
    stripe_object = util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )
    return stripe_object

async def auto_paging_iter_patch(self):
    page = self

    while True:
        if (
            "ending_before" in self._retrieve_params
            and "starting_after" not in self._retrieve_params
        ):
            for item in reversed(page):
                yield item
            page = await page.previous_page()
        else:
            for item in page:
                yield item
            page = await page.next_page()

        if page.is_empty:
            break

async def next_page_patch(
    self, api_key=None, stripe_version=None, stripe_account=None, **params
):
    if not self.has_more:
        return self.empty_list(
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )

    last_id = self.data[-1].id

    params_with_filters = self._retrieve_params.copy()
    params_with_filters.update({"starting_after": last_id})
    params_with_filters.update(params)

    return await self.list(
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        **params_with_filters
    )

async def previous_page_patch(
    self, api_key=None, stripe_version=None, stripe_account=None, **params
):
    if not self.has_more:
        return self.empty_list(
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )

    first_id = self.data[0].id

    params_with_filters = self._retrieve_params.copy()
    params_with_filters.update({"ending_before": first_id})
    params_with_filters.update(params)

    return await self.list(
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        **params_with_filters
    )


stripe.ListObject.list = list_patch
stripe.ListObject.create = create_patch
stripe.ListObject.retrieve = retrieve_patch
stripe.ListObject._request = _request_patch
stripe.ListObject.auto_paging_iter = auto_paging_iter_patch
stripe.ListObject.previous_page = previous_page_patch
stripe.ListObject.next_page = next_page_patch


import stripe


def modify_patch(cls, sid, **params):
    raise NotImplementedError(
        "Can't modify a reversal without a transfer"
        "ID. Call save on transfer.reversals.retrieve('reversal_id')"
    )


def retrieve_patch(cls, id, api_key=None, **params):
    raise NotImplementedError(
        "Can't retrieve a reversal without a transfer"
        "ID. Use transfer.reversals.retrieve('reversal_id')"
    )


stripe.Reversal.modify = classmethod(modify_patch)
stripe.Reversal.retrieve = classmethod(retrieve_patch)


import stripe
from async_stripe.api_resources.abstract import patch_nested_resources


async def usage_record_summaries_patch(self, **params):
    """usage_record_summaries is deprecated, use SubscriptionItem.list_usage_record_summaries instead."""
    return await self.request(
        "get", self.instance_url() + "/usage_record_summaries", params
    )


stripe.SubscriptionItem.usage_record_summaries = usage_record_summaries_patch


nested_resources = ["usage_record", "usage_record_summary"]

patch_nested_resources(stripe.SubscriptionItem, nested_resources)


import stripe
from stripe import oauth, six
from stripe import util
from stripe.six.moves.urllib.parse import quote_plus
from async_stripe.api_resources.abstract import (
    patch_custom_methods, patch_nested_resources
)


async def reject_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/reject"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def retrieve_patch(cls, id=None, api_key=None, **params):
    instance = cls(id, api_key, **params)
    await instance.refresh()
    return instance


async def modify_patch(cls, id=None, **params):
    url = cls._build_instance_url(id)
    return await cls._static_request("post", url, **params)


async def persons_patch(self, **params):
    return await self.request("get", self.instance_url() + "/persons", params)


async def deauthorize_patch(self, **params):
    params["stripe_user_id"] = self.id
    return await oauth.OAuth.deauthorize(**params)


stripe.Account.reject = reject_patch
stripe.Account.retrieve = classmethod(retrieve_patch)
stripe.Account.modify = classmethod(modify_patch)
stripe.Account.persons = persons_patch
stripe.Account.deauthorize = deauthorize_patch


def serialize_patch(self, previous):
    params = super(Account, self).serialize(previous)
    previous = previous or self._previous or {}

    for k, v in six.iteritems(self):
        if (
            k == "individual"
            and isinstance(v, stripe.api_resources.Person)
            and k not in params
        ):
            params[k] = v.serialize(previous.get(k, None))

    return params


nested_resources = ["capability", "login_link", "external_account", "person",]

patch_nested_resources(stripe.Account, nested_resources)


custom_methods = [
    {"name": "reject", "http_verb": "post"},
]

patch_custom_methods(stripe.Account, custom_methods)


import stripe


async def modify_patch(cls, sid, **params):
    raise NotImplementedError(
        "Can't update a capability without an account ID. Update a capability using "
        "account.modify_capability('acct_123', 'acap_123', params)"
    )

async def retrieve_patch(cls, id, api_key=None, **params):
    raise NotImplementedError(
        "Can't retrieve a capability without an account ID. Retrieve a capability using "
        "account.retrieve_capability('acct_123', 'acap_123')"
    )


stripe.Capability.modify = classmethod(modify_patch)
stripe.Capability.retrieve = classmethod(retrieve_patch)


import stripe
from stripe import error, version, util, six
from stripe.multipart_data_generator import MultipartDataGenerator
from stripe.six.moves.urllib.parse import urlencode

from stripe.api_requestor import APIRequestor
from stripe.api_requestor import _build_api_url, _api_encode

from async_stripe import http_client


def init_patch(
    self,
    key=None,
    client=None,
    api_base=None,
    api_version=None,
    account=None,
):
    self.api_base = api_base or stripe.api_base
    self.api_key = key
    self.api_version = api_version or stripe.api_version
    self.stripe_account = account

    self._default_proxy = None

    from stripe import verify_ssl_certs as verify
    from stripe import proxy

    # only use client if mockclient (for testing)
    # otherwise use Tornado client
    if client:
        if client.name == 'mockclient':
            self._client = client
    elif stripe.default_http_client and \
        (
            stripe.default_http_client.name == 'mockclient' or \
            isinstance(stripe.default_http_client, http_client.TornadoAsyncHTTPClient)
        ):

        self._client = stripe.default_http_client
        if proxy != self._default_proxy:
            warnings.warn(
                "stripe.proxy was updated after sending a "
                "request - this is a no-op. To use a different proxy, "
                "set stripe.default_http_client to a new client "
                "configured with the proxy."
            )
    else:
        # If the stripe.default_http_client has not been set by the user
        # yet, we'll set it here. This way, we aren't creating a new
        # HttpClient for every request.
        stripe.default_http_client = http_client.new_default_http_client(
            verify_ssl_certs=verify, proxy=proxy
        )
        self._client = stripe.default_http_client
        self._default_proxy = proxy


APIRequestor.__init__ = init_patch


async def request_patch(self, method, url, params=None, headers=None):
    rbody, rcode, rheaders, my_api_key = await self.request_raw(
        method.lower(), url, params, headers, is_streaming=False
    )
    resp = self.interpret_response(rbody, rcode, rheaders)
    return resp, my_api_key


APIRequestor.request = request_patch 


async def request_stream_patch(self, method, url, params=None, headers=None):
    stream, rcode, rheaders, my_api_key = await self.request_raw(
        method.lower(), url, params, headers, is_streaming=True
    )
    resp = self.interpret_streaming_response(stream, rcode, rheaders)
    return resp, my_api_key


APIRequestor.request_stream = request_stream_patch


async def request_raw_patch(self, method, url, params=None, supplied_headers=None, is_streaming=False):
    """
    Mechanism for issuing an API call
    """

    if self.api_key:
        my_api_key = self.api_key
    else:
        from stripe import api_key

        my_api_key = api_key

    if my_api_key is None:
        raise error.AuthenticationError(
            "No API key provided. (HINT: set your API key using "
            '"stripe.api_key = <API-KEY>"). You can generate API keys '
            "from the Stripe web interface.  See https://stripe.com/api "
            "for details, or email support@stripe.com if you have any "
            "questions."
        )

    abs_url = "%s%s" % (self.api_base, url)

    encoded_params = urlencode(list(_api_encode(params or {})))

    # Don't use strict form encoding by changing the square bracket control
    # characters back to their literals. This is fine by the server, and
    # makes these parameter strings easier to read.
    encoded_params = encoded_params.replace("%5B", "[").replace("%5D", "]")

    if method == "get" or method == "delete":
        if params:
            abs_url = _build_api_url(abs_url, encoded_params)
        post_data = None
    elif method == "post":
        if (
            supplied_headers is not None
            and supplied_headers.get("Content-Type")
            == "multipart/form-data"
        ):
            generator = MultipartDataGenerator()
            generator.add_params(params or {})
            post_data = generator.get_post_data()
            supplied_headers[
                "Content-Type"
            ] = "multipart/form-data; boundary=%s" % (generator.boundary,)
        else:
            post_data = encoded_params
    else:
        raise error.APIConnectionError(
            "Unrecognized HTTP method %r.  This may indicate a bug in the "
            "Stripe bindings.  Please contact support@stripe.com for "
            "assistance." % (method,)
        )

    headers = self.request_headers(my_api_key, method)
    if supplied_headers is not None:
        for key, value in six.iteritems(supplied_headers):
            headers[key] = value

    util.log_info("Request to Stripe api", method=method, path=abs_url)
    util.log_debug(
        "Post details",
        post_data=encoded_params,
        api_version=self.api_version,
    )

    if is_streaming:
        (
            rcontent,
            rcode,
            rheaders,
        ) = await self._client.request_stream_with_retries(
            method, abs_url, headers, post_data
        )
    else:
        rcontent, rcode, rheaders = await self._client.request_with_retries(
            method, abs_url, headers, post_data
        )
    
    util.log_info("Stripe API response", path=abs_url, response_code=rcode)
    util.log_debug("API response body", body=rcontent)

    if "Request-Id" in rheaders:
        request_id = rheaders["Request-Id"]
        util.log_debug(
            "Dashboard link for request",
            link=util.dashboard_link(request_id),
        )

    return rcontent, rcode, rheaders, my_api_key


APIRequestor.request_raw = request_raw_patch



import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def cancel_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/cancel"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def redact_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/redact"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.identity.VerificationSession.cancel = cancel_patch
stripe.identity.VerificationSession.redact = redact_patch


custom_methods = [
    {"name": "cancel", "http_verb": "post"},
    {"name": "redact", "http_verb": "post"},
]

patch_custom_methods(stripe.identity.VerificationSession, custom_methods)


from stripe import util
from stripe import api_requestor
from stripe.api_resources.abstract.listable_api_resource import (
    ListableAPIResource,
)


async def auto_paging_iter_patch(cls, *args, **params):
    obj = await cls.list(*args, **params)
    return obj.auto_paging_iter()


async def list_patch(
    cls, api_key=None, stripe_version=None, stripe_account=None, **params
):
    requestor = api_requestor.APIRequestor(
        api_key,
        api_base=cls.api_base(),
        api_version=stripe_version,
        account=stripe_account,
    )
    url = cls.class_url()
    response, api_key = await requestor.request("get", url, params)
    stripe_object = util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )
    stripe_object._retrieve_params = params
    return stripe_object


ListableAPIResource.auto_paging_iter = classmethod(auto_paging_iter_patch)
ListableAPIResource.list = classmethod(list_patch)


for subclass in ListableAPIResource.__subclasses__():
    subclass.auto_paging_iter = classmethod(auto_paging_iter_patch)
    subclass.list = classmethod(list_patch)



import stripe


async def modify_patch(cls, sid, **params):
    raise NotImplementedError(
        "Can't modify a bank account without a customer or account ID. "
        "Call save on customer.sources.retrieve('bank_account_id') or "
        "account.external_accounts.retrieve('bank_account_id') instead."
    )


async def retrieve_patch(
    cls,
    id,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    raise NotImplementedError(
        "Can't retrieve a bank account without a customer or account ID. "
        "Use customer.sources.retrieve('bank_account_id') or "
        "account.external_accounts.retrieve('bank_account_id') instead."
    )


stripe.BankAccount.modify = classmethod(modify_patch)
stripe.BankAccount.retrieve = classmethod(retrieve_patch)


import stripe
from stripe import api_requestor
from async_stripe.api_resources.abstract import patch_custom_methods


async def delete_discount_patch(self, **params):
    requestor = api_requestor.APIRequestor(
        self.api_key,
        api_version=self.stripe_version,
        account=self.stripe_account,
    )
    url = self.instance_url() + "/discount"
    _, api_key = await requestor.request("delete", url, params)
    self.refresh_from({"discount": None}, api_key, True)


stripe.Subscription.delete_discount = delete_discount_patch


custom_methods = [
    {"name": "delete_discount", "http_verb": "delete", "http_path": "discount"}
]

patch_custom_methods(stripe.Subscription, custom_methods)


import stripe
from stripe import api_requestor, connect_api_base


async def token_patch(api_key=None, **params):
    requestor = api_requestor.APIRequestor(
        api_key, api_base=connect_api_base
    )
    response, _ = await requestor.request("post", "/oauth/token", params, None)
    return response.data


async def deauthorize_patch(api_key=None, **params):
    requestor = api_requestor.APIRequestor(
        api_key, api_base=connect_api_base
    )
    stripe.OAuth._set_client_id(params)
    response, _ = await requestor.request(
        "post", "/oauth/deauthorize", params, None
    )
    return response.data


stripe.OAuth.token = staticmethod(token_patch)
stripe.OAuth.deauthorize = staticmethod(deauthorize_patch)


import stripe
from stripe import api_requestor, util


async def create_patch(
    cls,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    if "subscription_item" not in params:
        raise ValueError("Params must have a subscription_item key")

    subscription_item = params.pop("subscription_item")

    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    url = "/v1/subscription_items/%s/usage_records" % subscription_item
    headers = util.populate_headers(idempotency_key)
    response, api_key = await requestor.request("post", url, params, headers)

    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )


stripe.UsageRecord.create = classmethod(create_patch)


import textwrap

import stripe
from stripe import error, util
from stripe.http_client import HTTPClient, _now_ms

from tornado import gen
from tornado.httpclient import AsyncHTTPClient, HTTPError


def new_default_http_client(*args, **kwargs):
    impl = TornadoAsyncHTTPClient

    return impl(*args, **kwargs)


stripe.http_client.new_default_http_client = new_default_http_client


class TornadoAsyncHTTPClient(HTTPClient):
    name = "tornado_async_http_client"

    def __init__(self, request_timeout=80, **kwargs):
        super().__init__(**kwargs)
        self.request_timeout = request_timeout
        self.client = AsyncHTTPClient()

    async def request_with_retries(self, method, url, headers, post_data=None):
        return await self._request_with_retries_internal(
            method, url, headers, post_data, is_streaming=False
        )

    async def request_stream_with_retries(self, method, url, headers, post_data=None):
        return await self._request_with_retries_internal(
            method, url, headers, post_data, is_streaming=True
        )

    async def _request_with_retries_internal(self, method, url, headers, post_data, is_streaming):
        self._add_telemetry_header(headers)

        num_retries = 0

        while True:
            request_start = _now_ms()

            try:
                if is_streaming:
                    response = await self.request_stream(
                        method, url, headers, post_data
                    )
                else:
                    response = await self.request(method, url, headers, post_data)
                connection_error = None
            except error.APIConnectionError as e:
                connection_error = e
                response = None

            if self._should_retry(response, connection_error, num_retries):
                if connection_error:
                    util.log_info(
                        "Encountered a retryable error %s"
                        % connection_error.user_message
                    )
                num_retries += 1
                sleep_time = self._sleep_time_seconds(num_retries, response)
                util.log_info(
                    (
                        "Initiating retry %i for request %s %s after "
                        "sleeping %.2f seconds."
                        % (num_retries, method, url, sleep_time)
                    )
                )
                await gen.sleep(sleep_time)
            else:
                if response is not None:
                    self._record_request_metrics(response, request_start)

                    return response
                else:
                    raise connection_error

    async def request(self, method, url, headers, post_data=None):
        return await self._request_internal(
            method, url, headers, post_data, is_streaming=False
        )

    async def request_stream(self, method, url, headers, post_data=None):
        return await self._request_internal(
            method, url, headers, post_data, is_streaming=True
        )

    async def _request_internal(self, method, url, headers, post_data, is_streaming):
        try:
            response = await self.client.fetch(
                url, 
                method=method.upper(), 
                headers=headers, 
                body=post_data,
                request_timeout=self.request_timeout
            )
        except HTTPError as e:
            if e.response:
                content = e.response.body
                status_code = e.response.code
                headers = dict(e.response.headers)
            else:
                self._handle_request_error(e)
        except Exception as e:
            self._handle_request_error(e)
        else:
            if is_streaming:
                # :TODO:
                # Currently, the full response is downloaded at once
                # It will be better to actually stream response in chunks
                content = util.io.BytesIO(response.body)
            else:
                content = response.body
            status_code = response.code
            headers = dict(response.headers)
        return content, status_code, headers

    def _handle_request_error(self, e):
        msg = (
            "Unexpected error communicating with Stripe. "
            "If this problem persists, let us know at support@stripe.com."
        )
        msg = textwrap.fill(msg) + "\n\n(Network error: " + str(e) + ")"
        raise error.APIConnectionError(msg)

    def close(self):
        pass


# also patch base class
HTTPClient.request_with_retries = TornadoAsyncHTTPClient.request_with_retries
HTTPClient.request_stream_with_retries = TornadoAsyncHTTPClient.request_stream_with_retries
HTTPClient._request_with_retries_internal = TornadoAsyncHTTPClient._request_with_retries_internal


from __future__ import absolute_import, division, print_function

from stripe.api_resources.abstract.verify_mixin import VerifyMixin
from stripe import util


async def verify_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/verify"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


VerifyMixin.verify = verify_patch


from stripe import util
from stripe import api_requestor
from stripe.api_resources.abstract.api_resource import APIResource


async def _static_request_patch(
    cls,
    method_,
    url_,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    headers = util.populate_headers(idempotency_key)
    response, api_key = await requestor.request(method_, url_, params, headers)
    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )

APIResource._static_request = classmethod(_static_request_patch)


async def _static_request_stream_patch(
    cls,
    method_,
    url_,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    headers = util.populate_headers(idempotency_key)
    response, _ = await requestor.request_stream(method_, url_, params, headers)
    return response

APIResource._static_request_stream = classmethod(_static_request_stream_patch)


async def retrieve_patch(cls, id, api_key=None, **params):
    instance = cls(id, api_key, **params)
    await instance.refresh()
    return instance

APIResource.retrieve = classmethod(retrieve_patch)


async def refresh_patch(self):
    self.refresh_from(await self.request("get", self.instance_url()))
    return self

APIResource.refresh = refresh_patch



import stripe
from stripe import api_requestor
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def finalize_invoice_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/finalize"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def mark_uncollectible_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/mark_uncollectible"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def pay_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/pay"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def send_invoice_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/send"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def void_invoice_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/void"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def upcoming_patch(
    cls, api_key=None, stripe_version=None, stripe_account=None, **params
):
    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    url = cls.class_url() + "/upcoming"
    response, api_key = await requestor.request("get", url, params)
    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )


stripe.Invoice.finalize_invoice = finalize_invoice_patch
stripe.Invoice.mark_uncollectible = mark_uncollectible_patch
stripe.Invoice.pay = pay_patch
stripe.Invoice.send_invoice = send_invoice_patch
stripe.Invoice.void_invoice = void_invoice_patch
stripe.Invoice.upcoming = classmethod(upcoming_patch)


custom_methods = [
    {"name": "finalize_invoice", "http_verb": "post", "http_path": "finalize"},
    {"name": "mark_uncollectible", "http_verb": "post"},
    {"name": "pay", "http_verb": "post"},
    {"name": "send_invoice", "http_verb": "post", "http_path": "send"},
    {"name": "void_invoice", "http_verb": "post", "http_path": "void"},
]


patch_custom_methods(stripe.Invoice, custom_methods)



import stripe
from stripe import api_requestor
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def void_credit_note_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/void"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def preview_patch(
    cls, api_key=None, stripe_version=None, stripe_account=None, **params
):
    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    url = cls.class_url() + "/preview"
    response, api_key = await requestor.request("get", url, params)
    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )


stripe.CreditNote.void_credit_note = void_credit_note_patch
stripe.CreditNote.preview = classmethod(preview_patch)


custom_methods = [
    {"name": "void_credit_note", "http_verb": "post", "http_path": "void"},
]

patch_custom_methods(stripe.CreditNote, custom_methods)


from async_stripe.api_resources.abstract.api_resource import APIResource

from async_stripe.api_resources.abstract.verify_mixin import VerifyMixin

from async_stripe.api_resources.abstract.createable_api_resource import (
    CreateableAPIResource,
)
from async_stripe.api_resources.abstract.updateable_api_resource import (
    UpdateableAPIResource,
)
from async_stripe.api_resources.abstract.deletable_api_resource import (
    DeletableAPIResource,
)
from async_stripe.api_resources.abstract.listable_api_resource import (
    ListableAPIResource,
)

from async_stripe.api_resources.abstract.nested_resource_class_methods import (
    patch_nested_resources
)
from async_stripe.api_resources.abstract.custom_method import (
    patch_custom_methods
)



import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def attach_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/attach"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def detach_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/detach"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.PaymentMethod.attach = attach_patch
stripe.PaymentMethod.detach = detach_patch


custom_methods = [
    {"name": "attach", "http_verb": "post"},
    {"name": "detach", "http_verb": "post"},
]

patch_custom_methods(stripe.PaymentMethod, custom_methods)


import stripe
from stripe import util
from stripe.api_resources.abstract import ListableAPIResource
from async_stripe.api_resources.abstract import patch_nested_resources


async def refund_patch(self, idempotency_key=None, **params):
    headers = util.populate_headers(idempotency_key)
    url = self.instance_url() + "/refund"
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.ApplicationFee.refund = refund_patch


nested_resources = ["refund"]

patch_nested_resources(stripe.ApplicationFee, nested_resources)


from stripe import util
from stripe.stripe_object import StripeObject
from stripe import api_requestor


async def request_patch(self, method, url, params=None, headers=None):
    if params is None:
        params = self._retrieve_params

    requestor = api_requestor.APIRequestor(
        key=self.api_key,
        api_base=self.api_base(),
        api_version=self.stripe_version,
        account=self.stripe_account,
    )

    response, api_key = await requestor.request(method, url, params, headers)

    return util.convert_to_stripe_object(
        response, api_key, self.stripe_version, self.stripe_account
    )

StripeObject.request = request_patch


async def request_stream_patch(self, method, url, params=None, headers=None):
    if params is None:
        params = self._retrieve_params
    requestor = api_requestor.APIRequestor(
        key=self.api_key,
        api_base=self.api_base(),
        api_version=self.stripe_version,
        account=self.stripe_account,
    )
    response, _ = await requestor.request_stream(method, url, params, headers)

    return response

StripeObject.request_stream = request_stream_patch



import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def cancel_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/cancel"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def capture_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/capture"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def confirm_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/confirm"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.PaymentIntent.cancel = cancel_patch
stripe.PaymentIntent.capture = capture_patch
stripe.PaymentIntent.confirm = confirm_patch


custom_resources = [
    {"name": "cancel", "http_verb": "post"},
    {"name": "capture", "http_verb": "post"},
    {"name": "confirm", "http_verb": "post"},
]
patch_custom_methods(stripe.PaymentIntent, custom_resources)



import stripe


async def modify_patch(cls, fee, sid, **params):
    url = cls._build_instance_url(fee, sid)
    return await cls._static_request("post", url, **params)


async def retrieve_patch(cls, id, api_key=None, **params):
    raise NotImplementedError(
        "Can't retrieve a refund without an application fee ID. "
        "Use application_fee.refunds.retrieve('refund_id') instead."
    )


stripe.ApplicationFeeRefund.modify = classmethod(modify_patch)
stripe.ApplicationFeeRefund.retrieve = classmethod(retrieve_patch)


import stripe


async def modify_patch(cls, sid, **params):
    raise NotImplementedError(
        "Can't modify a person without an account"
        "ID. Call save on account.persons.retrieve('person_id')"
    )


async def retrieve_patch(cls, id, api_key=None, **params):
    raise NotImplementedError(
        "Can't retrieve a person without an account"
        "ID. Use account.persons.retrieve('person_id')"
    )


stripe.Person.modify = classmethod(modify_patch)
stripe.Person.retrieve = classmethod(retrieve_patch)


import stripe
from stripe import error
from stripe import util
from stripe.api_resources import Customer
from stripe.six.moves.urllib.parse import quote_plus
from async_stripe.api_resources.abstract import patch_nested_resources


async def detach_patch(self, idempotency_key=None, **params):
    token = util.utf8(self.id)

    if hasattr(self, "customer") and self.customer:
        extn = quote_plus(token)
        customer = util.utf8(self.customer)
        base = Customer.class_url()
        owner_extn = quote_plus(customer)
        url = "%s/%s/sources/%s" % (base, owner_extn, extn)
        headers = util.populate_headers(idempotency_key)

        self.refresh_from(await self.request("delete", url, params, headers))
        return self

    else:
        raise error.InvalidRequestError(
            "Source %s does not appear to be currently attached "
            "to a customer object." % token,
            "id",
        )


async def source_transactions_patch(self, **params):
    """source_transactions is deprecated, use Source.list_source_transactions instead."""
    return await self.request(
        "get", self.instance_url() + "/source_transactions", params
    )


stripe.Source.detach = detach_patch
stripe.Source.source_transactions = source_transactions_patch


nested_resources = ["source_transaction"]

patch_nested_resources(stripe.Source, nested_resources)


from stripe import util
from stripe.six.moves.urllib.parse import quote_plus
from stripe.api_resources.abstract.deletable_api_resource import (
    DeletableAPIResource,
)


async def _cls_delete_patch(cls, sid, **params):
    url = "%s/%s" % (cls.class_url(), quote_plus(util.utf8(sid)))
    return await cls._static_request("delete", url, **params)

@util.class_method_variant("_cls_delete")
async def delete_patch(self, **params):
    self.refresh_from(await self.request("delete", self.instance_url(), params))
    return self


DeletableAPIResource._cls_delete = classmethod(_cls_delete_patch)
DeletableAPIResource.delete = delete_patch


for subclass in DeletableAPIResource.__subclasses__():
    subclass._cls_delete = classmethod(_cls_delete_patch)
    subclass.delete = delete_patch


import stripe
from async_stripe.api_resources.abstract import patch_custom_methods


async def details_patch(self, idempotency_key=None, **params):
    return await self.request("get", self.instance_url() + "/details", params)


stripe.issuing.Card.details = details_patch


custom_methods = [
    {"name": "details", "http_verb": "get"}
]

patch_custom_methods(stripe.issuing.Card, custom_methods)


import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def pay_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/pay"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def return_order_patch(self, idempotency_key=None, **params):
    headers = util.populate_headers(idempotency_key)
    return await self.request(
        "post", self.instance_url() + "/returns", params, headers
    )


stripe.Order.pay = pay_patch
stripe.Order.return_order = return_order_patch


custom_methods = [
    {"name": "pay", "http_verb": "post"},
    {"name": "return_order", "http_verb": "post", "http_path": "returns"},
]

patch_custom_methods(stripe.Order, custom_methods)


import stripe
from stripe import util


async def modify_patch(cls, customer, id, **params):
    url = cls._build_instance_url(customer, id)
    return await cls._static_request("post", url, **params)


async def retrieve_patch(
    cls,
    id,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    raise NotImplementedError(
        "Can't retrieve an Alipay account without a customer ID. "
        "Use customer.sources.retrieve('alipay_account_id') instead."
    )


stripe.AlipayAccount.modify = classmethod(modify_patch)
stripe.AlipayAccount.retrieve = classmethod(retrieve_patch)


import stripe
from async_stripe.api_resources.abstract import (
    patch_nested_resources, patch_custom_methods
)
from stripe import api_requestor
from stripe import util


async def delete_discount_patch(self, **params):
    requestor = api_requestor.APIRequestor(
        self.api_key,
        api_version=self.stripe_version,
        account=self.stripe_account,
    )
    url = self.instance_url() + "/discount"
    _, api_key = await requestor.request("delete", url, params)
    self.refresh_from({"discount": None}, api_key, True)

async def list_payment_methods_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/payment_methods"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("get", url, params, headers))
    return self


stripe.Customer.delete_discount = delete_discount_patch
stripe.Customer.list_payment_methods = list_payment_methods_patch


netsted_resources = ['balance_transaction', 'source', 'tax_id']
patch_nested_resources(stripe.Customer, netsted_resources)


# always patch custom methods after patching other methods
custom_resources = [
    {"name": "delete_discount", "http_verb": "delete", "http_path": "discount"},
    {"name": "list_payment_methods", "http_verb": "get", "http_path": "payment_methods"},
]
patch_custom_methods(stripe.Customer, custom_resources)



from stripe import util
from stripe.six.moves.urllib.parse import quote_plus
from stripe.api_resources.abstract.updateable_api_resource import (
    UpdateableAPIResource,
)


async def modify_patch(cls, sid, **params):
    url = "%s/%s" % (cls.class_url(), quote_plus(util.utf8(sid)))
    return await cls._static_request("post", url, **params)


async def save_patch(self, idempotency_key=None):
    updated_params = self.serialize(None)
    headers = util.populate_headers(idempotency_key)

    if updated_params:
        self.refresh_from(
            await self.request(
                "post", self.instance_url(), updated_params, headers
            )
        )
    else:
        util.logger.debug("Trying to save already saved object %r", self)
    return self


UpdateableAPIResource.modify = classmethod(modify_patch)
UpdateableAPIResource.save = save_patch


for subclass in UpdateableAPIResource.__subclasses__():
    subclass.modify = classmethod(modify_patch)
    subclass.save = save_patch


from stripe import util
from stripe import api_requestor


def patch_nested_resources(cls, nested_resources):
    for resource in nested_resources:
        nested_resource_request_method = '%ss_request' % resource

        async def nested_resource_request(
            cls,
            method,
            url,
            api_key=None,
            idempotency_key=None,
            stripe_version=None,
            stripe_account=None,
            **params
        ):
            requestor = api_requestor.APIRequestor(
                api_key, api_version=stripe_version, account=stripe_account
            )
            headers = util.populate_headers(idempotency_key)
            response, api_key = await requestor.request(method, url, params, headers)
            return util.convert_to_stripe_object(
                response, api_key, stripe_version, stripe_account
            )
    
        setattr(cls, nested_resource_request_method, classmethod(nested_resource_request))


import stripe
from stripe import error
from stripe import util
from stripe.api_resources.abstract import DeletableAPIResource
from stripe.api_resources.abstract import UpdateableAPIResource
from stripe.api_resources.account import Account
from stripe.api_resources.customer import Customer
from stripe.api_resources.recipient import Recipient
from stripe.six.moves.urllib.parse import quote_plus


class Card(DeletableAPIResource, UpdateableAPIResource):
    OBJECT_NAME = "card"


async def modify_patch(cls, sid, **params):
    raise NotImplementedError(
        "Can't modify a card without a customer, recipient or account "
        "ID. Call save on customer.sources.retrieve('card_id'), "
        "recipient.cards.retrieve('card_id'), or "
        "account.external_accounts.retrieve('card_id') instead."
    )


async def retrieve_patch(
    cls,
    id,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    raise NotImplementedError(
        "Can't retrieve a card without a customer, recipient or account "
        "ID. Use customer.sources.retrieve('card_id'), "
        "recipient.cards.retrieve('card_id'), or "
        "account.external_accounts.retrieve('card_id') instead."
    )


stripe.Card.modify = classmethod(modify_patch)
stripe.Card.retrieve = classmethod(retrieve_patch)


import stripe
from stripe import util
from async_stripe.api_resources.abstract import (
    patch_custom_methods, patch_nested_resources
)


async def cancel_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/cancel"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.Transfer.cancel = cancel_patch


netsted_resources = ["reversal"]
patch_nested_resources(stripe.Transfer, netsted_resources)


custom_methods = [
    {"name": "cancel", "http_verb": "post"}
]
patch_custom_methods(stripe.Transfer, custom_methods)


import stripe
from stripe import util
from stripe import api_requestor
from stripe.six.moves.urllib.parse import quote_plus
from async_stripe.api_resources.abstract import patch_custom_methods


async def accept_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/accept"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def cancel_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/cancel"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def finalize_quote_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/finalize"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def list_computed_upfront_line_items_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/computed_upfront_line_items"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("get", url, params, headers))
    return self

async def list_line_items_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/line_items"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("get", url, params, headers))
    return self

async def _cls_pdf_patch(
    cls,
    sid,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    url = "%s/%s/%s" % (
        cls.class_url(),
        quote_plus(util.utf8(sid)),
        "pdf",
    )
    requestor = api_requestor.APIRequestor(
        api_key,
        api_base=stripe.upload_api_base,
        api_version=stripe_version,
        account=stripe_account,
    )
    headers = util.populate_headers(idempotency_key)
    response, _ = await requestor.request_stream("get", url, params, headers)
    return response

@util.class_method_variant("_cls_pdf")
async def pdf_patch(
    self,
    api_key=None,
    api_version=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    version = api_version or stripe_version
    requestor = api_requestor.APIRequestor(
        api_key,
        api_base=stripe.upload_api_base,
        api_version=version,
        account=stripe_account,
    )
    url = self.instance_url() + "/pdf"
    return await requestor.request_stream("get", url, params=params)


stripe.Quote.accept = accept_patch
stripe.Quote.cancel = cancel_patch
stripe.Quote.finalize_quote = finalize_quote_patch
stripe.Quote.list_computed_upfront_line_items = list_computed_upfront_line_items_patch
stripe.Quote.list_line_items = list_line_items_patch
stripe.Quote._cls_pdf = classmethod(_cls_pdf_patch)
stripe.Quote.pdf = pdf_patch


custom_resources = [
    {"name": "accept", "http_verb": "post"},
    {"name": "cancel", "http_verb": "post"},
    {"name": "finalize_quote", "http_verb": "post", "http_path": "finalize"},
    {"name": "list_computed_upfront_line_items", "http_verb": "get", "http_path": "computed_upfront_line_items"},
    {"name": "list_line_items", "http_verb": "get", "http_path": "line_items"},
]
patch_custom_methods(stripe.Quote, custom_resources)



from async_stripe.api_resources.abstract import *

from async_stripe.api_resources import list_object

from async_stripe.api_resources import checkout 

from async_stripe.api_resources import identity

from async_stripe.api_resources import issuing

from async_stripe.api_resources import account
from async_stripe.api_resources import alipay_account
from async_stripe.api_resources import application_fee
from async_stripe.api_resources import application_fee_refund
from async_stripe.api_resources import bank_account
from async_stripe.api_resources import capability
from async_stripe.api_resources import card
from async_stripe.api_resources import charge
from async_stripe.api_resources import credit_note
from async_stripe.api_resources import customer
from async_stripe.api_resources import dispute
from async_stripe.api_resources import ephermal_key
from async_stripe.api_resources import file
from async_stripe.api_resources import invoice
from async_stripe.api_resources import order
from async_stripe.api_resources import payment_intent
from async_stripe.api_resources import payment_method
from async_stripe.api_resources import payout
from async_stripe.api_resources import person
from async_stripe.api_resources import reversal
from async_stripe.api_resources import review
from async_stripe.api_resources import setup_intent
from async_stripe.api_resources import source
from async_stripe.api_resources import subscription
from async_stripe.api_resources import subscription_item
from async_stripe.api_resources import subscription_schedule
from async_stripe.api_resources import topup
from async_stripe.api_resources import transfer
from async_stripe.api_resources import usage_record
from async_stripe.api_resources import quote



from __future__ import absolute_import, division, print_function

import stripe
from stripe import api_requestor
from stripe import util


async def create_patch(
    # 'api_version' is deprecated, please use 'stripe_version'
    cls,
    api_key=None,
    api_version=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    version = api_version or stripe_version
    requestor = api_requestor.APIRequestor(
        api_key,
        api_base=stripe.upload_api_base,
        api_version=version,
        account=stripe_account,
    )
    url = cls.class_url()
    supplied_headers = {"Content-Type": "multipart/form-data"}
    response, api_key = await requestor.request(
        "post", url, params=params, headers=supplied_headers
    )
    return util.convert_to_stripe_object(
        response, api_key, version, stripe_account
    )


stripe.File.create = classmethod(create_patch)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['async_stripe',
 'async_stripe.api_resources',
 'async_stripe.api_resources.abstract',
 'async_stripe.api_resources.checkout',
 'async_stripe.api_resources.identity',
 'async_stripe.api_resources.issuing']

package_data = \
{'': ['*']}

install_requires = \
['stripe==2.61.0', 'tornado>=5.1']

setup_kwargs = {
    'name': 'async-stripe',
    'version': '2.61.0',
    'description': "An asynchronous wrapper around Stripe's official python library.",
    'long_description': "# async-stripe\n\n[![Build Status](https://travis-ci.com/bhch/async-stripe.svg?branch=master)](https://travis-ci.com/bhch/async-stripe)\n\nAn asynchronous wrapper around Stripe's official python library. \n\n## How it works\n\nasync-stripe monkey-patches the stripe library and replaces the synchronous \nhttp request methods with asynchronous methods.\n\nMonkey-patching allows us to avoid a complete rewrite and keep the usage api \nsimilar to the official library.\n\nThis **doesn't use threads**, but **actual async coroutines** and \n[non-blocking http client][1] to make requests.\n\n## Install\n\nInstall requires Python 3.6 or newer.\n\n```sh\n$ pip install async-stripe\n```\n\n## Usage\n\nThe usage api is similar to Stripe's official library:\n\n```python\nfrom async_stripe import stripe\n\nstripe.api_key = '<stripe-secret-key>'\n\npayment_intent = await stripe.PaymentIntent.create(amount=1000, currency='usd')\n\nprint(payment_intent.id)\n```\n\n---\n\n**/!\\ Note:** Since this library monkey-patches the actual `stripe` library, \nyou should avoid using the two in the same process.\n\nOnce you import `async_stripe`, the official `stripe` library gets patched with \nasync methods and the original synchronous api won't be available.\n\n---\n\n## Configuration\n\nPlease see [`stripe-python`'s README][5] file for configuring logging and other things.\n\n#### Unsupported configurations:\n\n`async-stripe` aims to be a drop-in async replacement for `stripe`. However, \nthere are a few things which are not yet supported:\n\n - Custom http client: Currently, it's not possible to configure a \n custom http client and Tornado's [`AsyncHTTPClient`][1] will be used by default.\n - Proxy: Connecting to api via a proxy is not supported yet.\n\n## Development and Testing\n\nWhen adding new features and monkey-patches, please add relevant tests and \nensure that all the tests also pass. \n\nIn most cases, you shouldn't need to write the tests yourself: you can just \ncopy-paste the tests form the original stripe library and change the synchronous \nmethods to asynchronous methods. \n\nFor testing, first, [install and run the `stripe-mock` api server][2].\n\nNext, install `pytest`, `pytest-mock` and `pytest-asyncio` python packages in \nyour virtualenv.\n\nFinally, run the tests like this:\n\n```sh\n$ pytest tests\n\n# or run a specific test\n$ pytest tests/api_resources/test_customer.py\n$ pytest tests/api_resources/test_customer.py::TestCustomer\n```\n\n## License\n\nA lot of the code (especially tests) are copied with slight modifications from \nStripe's official library. That code is licensed under \n[MIT License][3].\n\nRest of the original code is licensed under [BSD-3-Clause License][4].\n\n\n[1]: https://www.tornadoweb.org/en/stable/httpclient.html#tornado.httpclient.AsyncHTTPClient\n[2]: https://github.com/stripe/stripe-mock\n[3]: LICENSE.stripe.txt\n[4]: LICENSE.txt\n[5]: https://github.com/stripe/stripe-python/blob/master/README.md",
    'author': 'Bharat Chauhan',
    'author_email': 'tell.bhch@gmail.com',
    'maintainer': 'Bharat Chauhan',
    'maintainer_email': 'tell.bhch@gmail.com',
    'url': 'https://github.com/bhch/async-stripe',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from stripe import util
from stripe.six.moves.urllib.parse import quote_plus



def patch_custom_methods(cls, custom_resources):
    for resource in custom_resources:
        http_verb = resource["http_verb"]

        if http_verb not in ["get", "post", "delete"]:
            raise ValueError(
                "Invalid http_verb: %s. Must be one of 'get', 'post' or 'delete'"
                % http_verb
            )

        name = resource["name"]
        http_path = resource.get("http_path", name)
        is_streaming = resource.get("is_streaming", False)

        # move the actual patching to a separate function
        # to avoid "Late binding closures".
        # See: https://docs.python-guide.org/writing/gotchas/#late-binding-closures
        _patch(cls=cls, name=name, http_verb=http_verb, http_path=http_path, is_streaming=is_streaming)


def _patch(cls, name, http_verb, http_path, is_streaming):
    async def custom_method_request(cls, sid, **params):
        url = "%s/%s/%s" % (
            cls.class_url(),
            quote_plus(util.utf8(sid)),
            http_path,
        )
        return await cls._static_request(http_verb, url, **params)

    async def custom_method_request_stream(cls, sid, **params):
        url = "%s/%s/%s" % (
            cls.class_url(),
            quote_plus(util.utf8(sid)),
            http_path,
        )
        return await cls._static_request_stream(http_verb, url, **params)

    if is_streaming:
        class_method_impl = classmethod(custom_method_request_stream)
    else:
        class_method_impl = classmethod(custom_method_request)

    existing_method = getattr(cls, name, None)
    if existing_method is None:
        setattr(cls, name, class_method_impl)
    else:
        # If a method with the same name we want to use already exists on
        # the class, we assume it's an instance method. In this case, the
        # new class method is prefixed with `_cls_`, and the original
        # instance method is decorated with `util.class_method_variant` so
        # that the new class method is called when the original method is
        # called as a class method.
        setattr(cls, "_cls_" + name, class_method_impl)
        instance_method = util.class_method_variant("_cls_" + name)(
            existing_method
        )
        setattr(cls, name, instance_method)


import stripe

# register this library as a plugin
stripe.set_app_info('async-stripe', url='https://github.com/bhch/async-stripe')

# import monkey patches
from async_stripe import stripe_object
from async_stripe import http_client
from async_stripe import api_requestor
from async_stripe import api_resources
from async_stripe import oauth


import stripe
from stripe import api_requestor
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def capture_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/capture"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def refund_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/refund"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def update_dispute_patch(self, idempotency_key=None, **params):
    requestor = api_requestor.APIRequestor(
        self.api_key,
        api_version=self.stripe_version,
        account=self.stripe_account,
    )
    url = self.instance_url() + "/dispute"
    headers = util.populate_headers(idempotency_key)
    response, api_key = await requestor.request("post", url, params, headers)
    self.refresh_from({"dispute": response}, api_key, True)
    return self.dispute

async def close_dispute_patch(self, idempotency_key=None, **params):
    requestor = api_requestor.APIRequestor(
        self.api_key,
        api_version=self.stripe_version,
        account=self.stripe_account,
    )
    url = self.instance_url() + "/dispute/close"
    headers = util.populate_headers(idempotency_key)
    response, api_key = await requestor.request("post", url, params, headers)
    self.refresh_from({"dispute": response}, api_key, True)
    return self.dispute

async def mark_as_fraudulent_patch(self, idempotency_key=None):
    params = {"fraud_details": {"user_report": "fraudulent"}}
    url = self.instance_url()
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def mark_as_safe_patch(self, idempotency_key=None):
    params = {"fraud_details": {"user_report": "safe"}}
    url = self.instance_url()
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.Charge.capture = capture_patch
stripe.Charge.refund = refund_patch
stripe.Charge.update_dispute = update_dispute_patch
stripe.Charge.close_dispute = close_dispute_patch
stripe.Charge.mark_as_fraudulent = mark_as_fraudulent_patch
stripe.Charge.mark_as_safe = mark_as_safe_patch


custom_methods = [
    {"name": "capture", "http_verb": "post"},
]

patch_custom_methods(stripe.Charge, custom_methods)


import stripe
from stripe import api_requestor
from stripe import util


async def create_patch(
    cls,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    if stripe_version is None:
        raise ValueError(
            "stripe_version must be specified to create an ephemeral "
            "key"
        )

    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )

    url = cls.class_url()
    headers = util.populate_headers(idempotency_key)
    response, api_key = await requestor.request("post", url, params, headers)
    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )


stripe.EphemeralKey.create = classmethod(create_patch)